% This is a demo for the Supervised Spatio-Temporal Neighborhood Topology Learning (SSTNTL) algorithm.

% Please cite our paper, if you use this code for your project. Thanks.
% A J Ma, P C Yuen and J H Lai, "Supervised Spatio-Temporal Neighborhood Topology Learning for Action Recognition", 
% IEEE Trans. Circuits and Systems for Video Technology, 2013.

clear
clc

load data_pcawei

% parameter setting
sstntl_dim = 60;
para_ss = 0.06;
para_tpc = 0.9;
Knn = 1;

confusion_matrix = zeros(num_actions, num_actions);
number_correct = 0;

for ind_test_person = 1:num_persons % leave-one-out, 9 persons
    % leave n-th person out
    actions_test = actions(ind_test_person : num_persons : end);
    tag_test = tag_all(ind_test_person : num_persons : end);
    
    actions_train = actions;
    actions_train(ind_test_person : num_persons : end) = [];
    tag_train = tag_all;
    tag_train(ind_test_person : num_persons : end) = [];
    
    sstntl_mapping = sstntl(actions_train, tag_train, sstntl_dim, para_ss, para_tpc);
        
    actions_train = cell_multiply_by_matrix(actions_train, sstntl_mapping);
    actions_test = cell_multiply_by_matrix(actions_test, sstntl_mapping);
    
    tag = cell_median_hausdorff_nn(actions_train, tag_train, actions_test, Knn);
    
    number_correct = sum(tag_test==tag) + number_correct;
    
    for i = 1:length(tag_test)
        confusion_matrix(tag_test(i),tag(i)) = confusion_matrix(tag_test(i),tag(i)) + 1;
    end
end
regconition_rate = number_correct / length(tag_test) / num_persons;
