function sstntl_mapping = sstntl(actions_train, tag_train, sstntl_dim, para_ss, para_tpc)

pca_dim = size(actions_train{1,1},1);
uc = unique(tag_train);

% compute the Laplacian
XLX = zeros(pca_dim);
XDX = zeros(pca_dim);
XX = zeros(pca_dim);
for ind_action = 1:length(uc)
    X = cell2mat(actions_train(tag_train==uc(ind_action)));
    mat_dist = L2_distance(X, X);
    num_samples = sum(tag_train==uc(ind_action));
    % construct supervised spatial neighbors
    num_frames_all = size(X, 2);
    k_ssn = round(para_ss * num_frames_all);
    G_ssn = sparse(false(num_frames_all));
    [~, ind] = sort(mat_dist);
    for j = 1:num_frames_all
        G_ssn(ind(2:k_ssn+1, j), j) = 1;
    end
    G_ssn = max(G_ssn, G_ssn');
    
    % construct temporal pose correspondence neighbors
    G_tpcn = sparse(false(num_frames_all));
    ind_j = 0;    
    for j = 1:num_samples
        if para_tpc == 0
            break
        end
        action_j = actions_train(tag_train==uc(ind_action));
        action_j = action_j{1,j};
        num_frames_action_j = size(action_j, 2);
        ind_i = ind_j + num_frames_action_j;
        for i = j+1:num_samples
            action_i = actions_train(tag_train==uc(ind_action));
            action_i = action_i{1,i};
            num_frames_action_i = size(action_i, 2);
            
            % DTW
            SM = 1 - (action_j'*action_i)./(sqrt(sum(action_j.^2))'*sqrt(sum(action_i.^2)));
            [r,c] = size(SM);
            D = zeros(r+1, c+1);
            D(1,:) = NaN;
            D(:,1) = NaN;
            D(1,1) = 0;
            D(2:(r+1), 2:(c+1)) = SM;
            phi = zeros(r+1,c+1);            
            for ii = 2:r+1;
                for jj = 2:c+1;
                    kk1 = 2;
                    kk2 = 1;
                    dd = D(ii,jj);
                    [dmax, tb] = min([D(ii-1, jj-1)+dd, D(max(1,ii-2), jj-1)+dd*kk1, D(ii-1, ...
                        max(1,jj-2))+dd*kk1, D(ii-1,jj)+kk2*dd, D(ii,jj-1)+kk2*dd]);
                    D(ii,jj) = dmax;
                    phi(ii,jj) = tb;
                end
            end
            ii = r+1;
            jj = c+1;
            p = ii;
            q = jj;
            while ii > 2 && jj > 2
                tb = phi(ii,jj);
                if (tb == 1)
                    ii = ii-1;
                    jj = jj-1;
                elseif (tb == 2)
                    ii = ii-2;
                    jj = jj-1;
                elseif (tb == 3)
                    jj = jj-2;
                    ii = ii-1;
                elseif (tb == 4)
                    ii = ii-1;
                elseif (tb == 5)
                    jj = jj-1;
                end
                p = [ii,p];
                q = [jj,q];
            end
            p = p-1;
            q = q-1;
            % end DTW
            
            for k = 1:length(p)
                G_tpcn(ind_j+p(k), ind_i+q(k)) = 1;
            end
            ind_i = ind_i + num_frames_action_i;
        end
        ind_j = ind_j + num_frames_action_j;
    end
    G_tpcn = max(G_tpcn, G_tpcn');
    
    for j = 1:num_frames_all
        if para_tpc == 0
            break
        end
        tmp_j = X(:, j);
        tmp_j_neighbors = X(:, G_tpcn(j, :));
        ind_G_stn_j = find(G_tpcn(j, :));
        tmp = L2_distance(tmp_j, tmp_j_neighbors);
        [tmp ind] = sort(tmp);
        k_tpc = round(para_tpc * length(ind));
        G_tpcn(j, ind_G_stn_j(ind(k_tpc+1:end))) = 0;
    end
    G_tpcn = max(G_tpcn, G_tpcn');
    
    % compute weight matrix W
    % simple
    W = max(G_ssn, G_tpcn);
    D = diag(sum(W));
    L = D - W;
    
    XLX = XLX + X * L * X';
    XDX = XDX + X * D * X';
    XX = XX + X * X';
end

[eigvector, eigvalue] = eig(XLX, XDX);
% Sort eigenvalues in descending order and get smallest eigenvectors
[~, ind] = sort(diag(eigvalue));
sstntl_mapping = eigvector(:,ind(1:sstntl_dim));
